
from catalog import ImageCatalog, Sequence
# from imageengine import Toto
from imageengine import IImageEngine
from imageengine import PixelType
from preprocessing import WhiteEstimator

def remove_traps(sequence, channel_id, trap_aabb):
    """Remove the traps in the input sequence

    :param Sequence sequence:
    :param str channel_id:
    :param Aabb trap_aabb:

    the idea of this method is as follows:
    1. allow the user to provide an axis-aligned bounding box that contains a trap, and use this area as a pattern to be searched in all images of the sequence. Provided the tolerance is big enough, this should find all traps in the sequence.
    2. compute a clean (without globules) trap, by computing for each pixel, the median value from all traps for that pixel.
    3. For each trap location, remove the trap by substracting the clean trap. This should leave the globules only.

    At the end of this image processing, we expect to only have the globules in the image.
    """
    white_estimator = WhiteEstimator(open_size=75, close_size=75, average_size=75)
    white_estimate = white_estimator.estimate_white([sequence], [channel_id])
    print(white_estimate)

    uniform_stack = IImageEngine.get_instance().create_hyperstack(width=white_estimate.width(), height=white_estimate.height(), num_slices=1, num_frames=sequence.num_frames, num_channels=1, pixel_type=PixelType.F32)

    ie = IImageEngine.get_instance()

    for frame_index in range(sequence.num_frames):
        non_uniform_frame = IImageEngine.get_instance().load_image(sequence.get_image_file_path(channel_index=0, frame_index=frame_index, slice_index=0))
        uniform_frame = IImageEngine.get_instance().divide(non_uniform_frame, white_estimate)
        uniform_stack.set_image(frame_index=frame_index, image=uniform_frame)

    first_image = uniform_stack.get_image(frame_index=0)
    trap_image = first_image.get_subimage(trap_aabb)

    ie.save_as_tiff(trap_image, '/home/graffy/Desktop/template.tiff')

    similarity_image = ie.match_template(first_image, trap_image)

    ie.save_as_tiff(similarity_image, '/home/graffy/Desktop/similarity.tiff')

    matches = ie.find_maxima(similarity_image)
    num_traps = len(matches)
    print("number of traps found : %d" % num_traps)
    num_expected_traps = 13  # 13 traps are completely visible in the 
    assert abs(num_traps - num_expected_traps) <= 1
    
    #non_uniform_stack = sequence.as_stack()
    #uniform_stack = IImageEngine.get_instance().divide(non_uniform_stack, white_estimate)
    # IImageEngine.get_instance().save_as_tiff(white_estimate, './white_estimate.tiff')
    